@@ -87,7 +87,7 @@ Future->wait_all(
                         # Redis might send the new item to a worker before we get the
                         # response back from the XADD command.
                         $pending->{$id} = 1;
-                        $primary->xadd(
+                        also => $primary->xadd(
                             # * means 'just use an autogenerated ID', which is fine for our needs
                             example_stream => '*',
                             id => $id,
@@ -97,7 +97,6 @@ Future->wait_all(
                             delete $pending->{$id}
                         })
                          ->retain
-                         ->without_cancel
                      }
                 )
             } while => sub { $active }
